#include <SI_EFM8BB1_Register_Enums.h>
#include "InitDevice.h"

extern void enter_DefaultMode_from_RESET(void) {
	WDT_0_enter_DefaultMode_from_RESET();
	PORTS_0_enter_DefaultMode_from_RESET();
	PORTS_1_enter_DefaultMode_from_RESET();
	PBCFG_0_enter_DefaultMode_from_RESET();
	CLOCK_0_enter_DefaultMode_from_RESET();
	PCA_0_enter_DefaultMode_from_RESET();
	PCACH_0_enter_DefaultMode_from_RESET();
	PCACH_1_enter_DefaultMode_from_RESET();
	PCACH_2_enter_DefaultMode_from_RESET();
	TIMER16_2_enter_DefaultMode_from_RESET();
	INTERRUPT_0_enter_DefaultMode_from_RESET();
}

extern void WDT_0_enter_DefaultMode_from_RESET(void) {
	//Disable watchdog with key sequence
	WDTCN = 0xDE; //First key
	WDTCN = 0xAD; //Second key

}

extern void PORTS_0_enter_DefaultMode_from_RESET(void) {
	P0MDOUT = P0MDOUT_B7__PUSH_PULL;
	//Skip all pins except 7
	P0SKIP = 0x7F;
}

extern void PORTS_1_enter_DefaultMode_from_RESET(void) {
	P1MDOUT = P1MDOUT_B0__PUSH_PULL | P1MDOUT_B1__PUSH_PULL;
	//Skip all pins except 0 and 1
	P1SKIP = 0xFC;

	//Leave P2.0 high for open-drain mode to work as input
	P2 = 1;
}

extern void PBCFG_0_enter_DefaultMode_from_RESET(void) {
	XBR1 = XBR1_PCA0ME__CEX0_CEX1_CEX2;
	XBR2 = XBR2_WEAKPUD__PULL_UPS_ENABLED | XBR2_XBARE__ENABLED;
}

extern void CLOCK_0_enter_DefaultMode_from_RESET(void) {
	//Select internal high-frequency oscillator without prescaler
	CLKSEL = CLKSEL_CLKSL__HFOSC;
}

extern void PCA_0_enter_DefaultMode_from_RESET(void) {
	//PCA off
	PCA0CN0_CR = PCA0CN0_CR__STOP;

	// $[PCA0MD - PCA mode]
	/*
	 // CIDL (PCA counter/timer idle control) = NORMAL (PCA continues to
	 //     function normally while the system controller is in idle mode.)
	 // ECF (PCA counter/timer overflow interrupt enable) = OVF_INT_DISABLED
	 //     (Disable the CF interrupt.)
	 // CPS (PCA counter/timer pulse select) = SYSCLK_DIV_12 (System clock divided by 12.)
	 */
	PCA0MD = PCA0MD_CIDL__NORMAL | PCA0MD_ECF__OVF_INT_DISABLED
			| PCA0MD_CPS__SYSCLK_DIV_12;

	//Invert polarity for correct PWM work
	PCA0POL = PCA0POL_CEX0POL__INVERT | PCA0POL_CEX1POL__INVERT
			| PCA0POL_CEX2POL__INVERT;
	//PCA on
	PCA0CN0_CR = PCA0CN0_CR__RUN;
}

extern void PCACH_0_enter_DefaultMode_from_RESET(void) {
	//PCA0 settings save
	uint8_t PCA0CN0_CR_save = PCA0CN0 & PCA0CN0_CR__BMASK;
	uint8_t PCA0PWM_ARSEL_save = PCA0PWM & PCA0PWM_ARSEL__BMASK;

	//Turn PCA0 off while updating PCACH0
	PCA0CN0 &= ~PCA0CN0_CR__BMASK;
	//Select capture/compare register)
	PCA0PWM &= ~PCA0PWM_ARSEL__BMASK;
	//PCA0 settings save

	//PCA0CPM0 - PCA channel 0 capture/compare mode]
	/*
	 // ECOM (channel 0 comparator function enable) = ENABLED (enable
	 //     comparator function.)
	 // PWM (channel 0 pulse width modulation mode enable) = ENABLED (enable
	 //     PWM function.)
	 */
	PCA0CPM0 = PCA0CPM0_ECOM__ENABLED | PCA0CPM0_PWM__ENABLED;

	//PCA0 settings restore
	PCA0CN0 |= PCA0CN0_CR_save;
	PCA0PWM |= PCA0PWM_ARSEL_save;
}

extern void PCACH_1_enter_DefaultMode_from_RESET(void) {
	//PCA0 settings save
	uint8_t PCA0CN0_CR_save = PCA0CN0 & PCA0CN0_CR__BMASK;
	uint8_t PCA0PWM_ARSEL_save = PCA0PWM & PCA0PWM_ARSEL__BMASK;

	//Turn PCA0 off while updating PCACH1
	PCA0CN0 &= ~PCA0CN0_CR__BMASK;
	// Select capture/compare register)
	PCA0PWM &= ~PCA0PWM_ARSEL__BMASK;

	//PCA0CPM1 - PCA channel 1 capture/compare mode]
	/*
	 // ECOM (channel 1 comparator function enable) = ENABLED (enable
	 //     comparator function.)
	 // PWM (channel 1 pulse width modulation mode enable) = ENABLED (enable
	 //     PWM function.)
	 */
	PCA0CPM1 = PCA0CPM1_ECOM__ENABLED
			| PCA0CPM1_PWM__ENABLED;

	//PCA0 settings restore
	PCA0CN0 |= PCA0CN0_CR_save;
	PCA0PWM |= PCA0PWM_ARSEL_save;
}

extern void PCACH_2_enter_DefaultMode_from_RESET(void) {
	//PCA0 settings save
	uint8_t PCA0CN0_CR_save = PCA0CN0 & PCA0CN0_CR__BMASK;
	uint8_t PCA0PWM_ARSEL_save = PCA0PWM & PCA0PWM_ARSEL__BMASK;

	// Turn PCA0 off while updating PCACH1
	PCA0CN0 &= ~PCA0CN0_CR__BMASK;
	// Select capture/compare register)
	PCA0PWM &= ~PCA0PWM_ARSEL__BMASK;
	// [PCA0 Settings Save]$

	//PCA0CPM1 - PCA channel 1 capture/compare mode]
	/*
	 // ECOM (channel 1 comparator function enable) = ENABLED (enable
	 //     comparator function.)
	 // PWM (channel 1 pulse width modulation mode enable) = ENABLED (enable
	 //     PWM function.)
	 */
	PCA0CPM2 = PCA0CPM2_ECOM__ENABLED
			| PCA0CPM2_PWM__ENABLED;
	//PCA0 settings restore
	PCA0CN0 |= PCA0CN0_CR_save;
	PCA0PWM |= PCA0PWM_ARSEL_save;
}

extern void TIMER16_2_enter_DefaultMode_from_RESET() {
	//Timer initialization, update frequency  125Hz
	//Save timer configuration
	uint8_t TMR2CN0_TR2_save = TMR2CN0 & TMR2CN0_TR2__BMASK;
	//Stop timer
	TMR2CN0 &= ~(TMR2CN0_TR2__BMASK);

	//Reload value for overflow frequency of 125Hz
	//Timer counts up, internal frequency is sysclk/12, result is
	//65536-(25000000/12/125)
	TMR2RLH = 0xBE;
	TMR2RLL = 0xE4;

	//Start timer
	TMR2CN0 |= TMR2CN0_TR2__RUN;

	//Timer restoration
	TMR2CN0 |= TMR2CN0_TR2_save;
}

extern void INTERRUPT_0_enter_DefaultMode_from_RESET() {
	//Enable all interrupts in general and timer 2
	IE = IE_EA__ENABLED | IE_ET2__ENABLED;
}
