/*
 * hal.c
 *
 *  Created on: 20.10.2018.
 *      Author: Petr
 */

#include "hal.h"
#include <SI_EFM8BB1_Register_Enums.h>
//Inversed state of button: 0 indicates pressed, non-zero indicated released
int buttState = 0;
//State machine for detecting button presses
int buttFSM = 0;
volatile uint32_t timer = 0, eventTime = 0, buttPressedTime = 0,
		buttReleasedTime = 0;

void setPWM(int channel, int val) {
	switch (channel) {
	case 0:
		PCA0CPH0 = val;
		break;
	case 1:
		PCA0CPH1 = val;
		break;
	case 2:
		PCA0CPH2 = val;
		break;
	default:
		break;
	}
}

void delay(uint32_t ms) {
	uint32_t targetMs = timer + (ms >> 3);
	while (timer < targetMs) {};
}

void setColor(uint32_t rgb) {
	setPWM(RGB_B, rgb & 255);
	rgb >>= 8;
	setPWM(RGB_G, rgb & 255);
	rgb >>= 8;
	setPWM(RGB_R, rgb);
}

int buttonPressed() {
	if (buttState != buttFSM) {
		buttFSM = buttState;
		return !buttState;
	}
	buttFSM = buttState;
	return 0;
}

//Returns zero if button is not pressed
int buttonState() {
	return !buttState;
}

uint32_t getButtonPressedTime() {
	return buttPressedTime << 3;
}

uint32_t getButtonReleasedTime() {
	return buttReleasedTime << 3;
}
int newButtState = 0;
uint32_t time = 0;
SI_INTERRUPT (TIMER2_ISR, TIMER2_IRQn) {
	TMR2CN0_TF2H = 0;                  // Clear Timer2 interrupt flag
	timer++;
	newButtState = P2 & 1;
	if (buttState != newButtState) {
		buttState = newButtState;
		//If timer overflow occured, inverse event time
		if (timer < eventTime)
			eventTime = -eventTime;
		time = timer - eventTime;
		//button is pressed
		if (!buttState) {
			buttReleasedTime = time;
			buttPressedTime = 0;
		} else {
			buttPressedTime = time;
			buttReleasedTime = 0;
		}
		eventTime = timer;
	}
}
